//
// Authenticator.java - manages the user registry, if there is one
//

package nyangau.se;

//...sincludes:0:
import java.util.Set;
import java.util.HashSet;
import java.util.Map;
import java.util.HashMap;
//...e

public class Authenticator
  {
  // local registry
  protected String realm;
  protected Set<String> userids;
  protected Map<String,String> passwords;
  // pre-authentication feature
  protected String preAuthSecretHeaderName;
  protected String preAuthSecretHeaderValue;
  protected String preAuthUseridHeaderName;
  // authorisation
  protected Map< String,Set<String> > userRoles;
  protected Map< String,Set<String> > roleSecureURIPs;
  protected String[] secureURIPs;
//...sUserRegistry:2:
public Authenticator(ServletContext sc)
  {
  if ( (realm = sc.getInitParameter("realm")) == null )
    realm = "nyangau.se";

  passwords = new HashMap<String,String>();
  userRoles = new HashMap<String,Set<String>>();

  userids = new HashSet<String>();
  String useridList;
  if ( (useridList = sc.getInitParameter("userids")) != null &&
       !useridList.equals("") )
    for ( String userid : useridList.split(",") )
      {
      String password;
      if ( (password = sc.getInitParameter("password."+userid)) != null )
        passwords.put(userid, password);
      String userRoleList;
      if ( (userRoleList = sc.getInitParameter("roles."+userid)) != null )
        {
        Set<String> userRoleSet = new HashSet<String>();
        for ( String role : userRoleList.split(",") )
          userRoleSet.add(role);
        userRoles.put(userid, userRoleSet);
        }
      }

  roleSecureURIPs = new HashMap<String,Set<String>>();
  String roleList;
  if ( (roleList = sc.getInitParameter("roles")) != null &&
       !roleList.equals("") )
    for ( String role : roleList.split(",") )
      {
      String secureURIPList;
      if ( (secureURIPList = sc.getInitParameter("secureURIs."+role)) != null )
        {
        Set<String> secureURIPSet = new HashSet<String>();
        for ( String secureURIP : secureURIPList.split(",") )
          secureURIPSet.add(secureURIP);
        roleSecureURIPs.put(role, secureURIPSet);
        }
      }

  String secureUriList;
  if ( (secureUriList = sc.getInitParameter("secureURIs")) != null &&
       !secureUriList.equals("") )
    secureURIPs = secureUriList.split(",");
  else
    secureURIPs = new String[0];

  preAuthSecretHeaderName  = sc.getInitParameter("preauth.secret.header.name");
  preAuthSecretHeaderValue = sc.getInitParameter("preauth.secret.header.value");
  preAuthUseridHeaderName  = sc.getInitParameter("preauth.userid.header.name");
  }
//...e
//...sgetRealm:2:
public String getRealm()
  {
  return realm;
  }
//...e
//...scheckPassword:2:
public boolean checkPassword(String userid, String password)
  {
  String p = passwords.get(userid);
  return p != null && p.equals(password);
  }
//...e
//...sisPreAuthConfigured:2:
public boolean isPreAuthConfigured()
  {
  return preAuthSecretHeaderName  != null &&
         preAuthSecretHeaderValue != null &&
         preAuthUseridHeaderName  != null ;
  }
//...e
//...sgetPreAuthSecretHeaderName:2:
public String getPreAuthSecretHeaderName()
  {
  return preAuthSecretHeaderName;
  }
//...e
//...sgetPreAuthSecretHeaderValue:2:
public String getPreAuthSecretHeaderValue()
  {
  return preAuthSecretHeaderValue;
  }
//...e
//...sgetPreAuthUseridHeaderName:2:
public String getPreAuthUseridHeaderName()
  {
  return preAuthUseridHeaderName;
  }
//...e
//...sisRequired:2:
public boolean isRequired(String uri)
  {
  for ( String secureURIP : secureURIPs )
    if ( URIPMatchesURI(secureURIP, uri) )
      return true;
  return false;
  }
//...e
//...sURIPMatchesURI:2:
protected static boolean URIPMatchesURI(String urip, String uri)
  {
  return ( urip.endsWith("*") && uri.startsWith(urip.substring(0,urip.length()-1)) )
         || urip.equals(uri);
  }
//...e
//...scheckURI:2:
protected boolean checkURI(Set<String> roleSet, String uri)
  {
  for ( String role : roleSet )
    {
    Set<String> secureURIPSet;
    if ( (secureURIPSet = roleSecureURIPs.get(role)) != null )
      {
      for ( String secureURIP : secureURIPSet )
        if ( URIPMatchesURI(secureURIP, uri) )
          return true;
      }
    }
  return false;
  }

public boolean checkURI(String userid, String uri)
  {
  Set<String> roleSet;
  if ( (roleSet = userRoles.get(userid)) != null )
    return checkURI(roleSet, uri);
  else if ( (roleSet = userRoles.get("default")) != null )
    return checkURI(roleSet, uri);
  else
    return false;
  }
//...e
//...sisUserInRole:2:
public boolean isUserInRole(String userid, String role)
  {
  Set<String> roleSet;
  if ( (roleSet = userRoles.get(userid)) != null )
    return roleSet.contains(role);
  else if ( (roleSet = userRoles.get("default")) != null )
    return roleSet.contains(role);
  else
    return false;
  }
//...e
  }
