//
// Connection.java - Connection
//

package nyangau.se;

//...simports:0:
import java.net.*;
import java.io.*;
import java.util.*;
import java.security.*;
import javax.net.*;
import javax.net.ssl.*;
import java.lang.management.*;
import javax.management.*;
//...e

public class Connection extends Thread
  {
  protected ServletEngine se;
  protected Socket socket;
  protected String scheme;
  protected int port; 
//...sConnection:2:
public Connection(ServletEngine se, Socket socket, String scheme, int port)
  {
  this.se     = se;
  this.socket = socket;
  this.scheme = scheme;
  this.port   = port;
  }
//...e
//...stryServeStatic:2:
protected boolean tryServeStatic(
  String uri,
  HttpServletResponse resp, ServletOutputStream os
  )
  throws IOException
  {
  InputStream is = null;
  if ( (is = se.getServletContext().getResourceAsStream(uri)) == null )
    return false;
  try
    {
    String mime;
    if ( (mime = se.getServletContext().getMimeType(uri)) == null )
      mime = "text/plain";
    resp.setContentType(mime);
    byte[] buf = new byte[0x10000];
    int n;
    while ( (n = is.read(buf)) > 0 )
      os.write(buf, 0, n);
    }
  finally
    {
    try
      {
      is.close();
      }
    catch ( IOException e )
      {
      se.log("Error closing input stream");
      }
    }
  return true;
  }
//...e
//...stryServeDynamic:2:
protected boolean tryServeDynamic(
  HttpServletRequest req, String uri,
  HttpServletResponse resp, ServletOutputStream os
  )
  throws jakarta.servlet.ServletException, IOException
  {
  jakarta.servlet.http.HttpServlet servlet = null;
  Vector<RegisteredServlet> rs = se.getRegisteredServlets();
  for ( int i = 0; i < rs.size(); i++ )
    {
    RegisteredServlet m = rs.elementAt(i);
    if ( uri.startsWith(m.getName()) )
      {
      servlet = m.getServlet();
      servlet.service(req, resp);
      return true;
      }
    }
  return false;
  }
//...e
//...srun:2:
// Internet Explorer has a bug in its handling of 302 Redirects.
// KB1933489 describes this bug on IE4 and IE5, but we've seen it on IE6 also.
// If the HTTP headers are delivered in a different TCP packet to the
// body of the reply, it fails to follow the redirect.
// So we ensure the ServletOutputStream is buffered.
// This allows the entire redirect to be buffered, and then sent down the
// wire in a single larger packet.
// Luckily, we are allowed to buffer the response.

//...sauthenticate:2:
//...sdecode:2:
protected static byte[] decode(String s)
  {
  String base64 = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/=";
  int i = 0, j = 0;
  byte b[] = new byte[(s.length()/4)*3];
    // Every 4 characters input can produce 3 (or less) output
  while ( i + 4 <= s.length() )
    {
    int v0 = base64.indexOf(s.charAt(i++));
    int v1 = base64.indexOf(s.charAt(i++));
    int v2 = base64.indexOf(s.charAt(i++));
    int v3 = base64.indexOf(s.charAt(i++));
    if ( v0 == -1 || v1 == -1 || v2 == -1 || v3 == -1 )
      return null; // base64 encoded data contains bad character
    if ( v0 == 64 || v1 == 64 )
      break;
    b[j++] = (byte) (  (v0<<2)|(v1>>4)         );
    if ( v2 == 64 )
      break;
    b[j++] = (byte) ( ((v1<<4)|(v2>>2)) & 0xff );
    if ( v3 == 64 )
      break;
    b[j++] = (byte) ( ((v2<<6)| v3    ) & 0xff );
    }
  if ( i < s.length() )
    return null; // Encoded data must be a multiple of 4 characters
  if ( j < b.length )
    {
    byte b2[] = new byte[j];
    System.arraycopy(b, 0, b2, 0, j);
    b = b2;
    }
  return b;
  }
//...e

protected String authenticate(Authenticator a, HttpServletRequest req)
  {
  // look for pre-authenticated request
  if ( a.isPreAuthConfigured() )
    {
    String shn = a.getPreAuthSecretHeaderName();
    String shv = a.getPreAuthSecretHeaderValue();
    String uhn = a.getPreAuthUseridHeaderName();
    String shva = req.getHeader(shn);
    if ( shva != null && shv.equals(shva) )
      {
      String u;
      if ( (u = req.getHeader(uhn)) != null )
        {
        req.attachUserid(u, HttpServletRequest.PRE_AUTH, a);
        return u;
        }
      }
    }
  // normal HTTP basic authentication
  String az;
  if ( (az = req.getHeader("Authorization")) == null )
    return null;
  String[] aza = az.split(" ");
  if ( aza.length < 2 )
    return null;
  if ( ! aza[0].trim().toLowerCase().equals("basic") )
    return null;
  byte[] b = decode(aza[1]);
  if ( b == null )
    return null;
  String up;
  try
    {
    up = new String(b, "UTF-8");
    }
  catch ( UnsupportedEncodingException e )
    {
    return null;
    }
  int iColon;
  if ( (iColon = up.indexOf(":")) == -1 )
    return null;
  String u = up.substring(0, iColon);
  String p = up.substring(iColon+1);
  if ( ! a.checkPassword(u, p) )
    return null;
  req.attachUserid(u, HttpServletRequest.BASIC_AUTH, a);
  return u;
  }
//...e

public void run()
  {
  String uri = "unknown";
  try
    {
    InputStream is = socket.getInputStream();
    HttpServletRequest req = new HttpServletRequest(is, scheme, port);
    BufferedOutputStream bos = new BufferedOutputStream(socket.getOutputStream());
    ServletOutputStream os = new ServletOutputStream(bos);
    HttpServletResponse resp = new HttpServletResponse(se, req, os);
    uri = req.getRequestURI();
    // getRequestURI() doesn't return URL decoded data
    // and yet we must look for content based upon URL decoded data
    uri = URLDecoder.decode(uri, "UTF-8");
    Authenticator a = se.getAuthenticator();
    String userid = null;
    if ( ! a.isRequired(uri) ||
         ( (userid = authenticate(a, req)) != null && a.checkURI(userid, uri) ) )
      {
      if ( tryServeStatic(uri, resp, os) )
        ; // successfully served static content
      else if ( tryServeDynamic(req, uri, resp, os) )
        ; // successfully served dynamic content
      else
        resp.sendError(HttpServletResponse.SC_NOT_FOUND);
      }
    else
      {
      resp.setStatus(HttpServletResponse.SC_UNAUTHORIZED);
      resp.addHeader("WWW-Authenticate", "Basic realm=\""+a.getRealm()+"\"");
      }
    os.flush();
    os.close();
    }
  catch ( jakarta.servlet.ServletException e )
    {
    se.log("Error serving page: "+uri+": "+e.toString());
    }
  catch ( EOFException e )
    {
    // Ignore connections that don't contain valid requests
    }
  catch ( IOException e )
    {
    se.log("Error serving page: "+uri+": "+e.toString());
    }
  try
    {
    socket.close();
    }
  catch ( IOException e2 )
    {
    se.log("Error closing socket: "+e2.toString());
    }
  }
//...e
  }
