//
// HttpServletRequest.java
//

package nyangau.se;

import java.net.*;
import java.io.*;
import java.util.*;

public class HttpServletRequest
  extends ServletRequest
  implements jakarta.servlet.http.HttpServletRequest
  {
  protected String authType;
  protected String method;
  protected String uri;
  protected String query;
  protected boolean rh;
  protected Dictionary<String,Vector<String>> headers = new Hashtable<String,Vector<String>>();
  protected String userid; // or null, if not authenticated
  protected Authenticator authenticator; // or null, if not authenticated
  // interface jakarta.servlet.http.HttpServletRequest
//...sgetAuthType:2:
public String getAuthType()
  {
  return authType;
  }
//...e
//...sgetMethod:2:
public String getMethod()
  {
  return method;
  }
//...e
//...sgetRequestURI:2:
public String getRequestURI()
  {
  return uri;
  } 
//...e
//...sgetQueryString:2:
public String getQueryString()
  {
  return query;
  }
//...e
//...sgetHeader:2:
public String getHeader(String h)
  {
  Vector<String> v;
  if ( (v = headers.get(h)) != null )
    return v.elementAt(0);
  else if ( (v = headers.get(h.toLowerCase())) != null )
    return v.elementAt(0);
  else
    return null;
  }
//...e
//...sgetIntHeader:2:
public int getIntHeader(String h)
  throws NumberFormatException
  {
  String v = getHeader(h);
  if ( v == null )
    return -1;
  return Integer.parseInt(v);
  }
//...e
//...sgetHeaders:2:
public Enumeration<String> getHeaders(String h)
  {
  Vector<String> v;
  if ( (v = headers.get(h)) != null )
    return v.elements();
  else
    return null;
  }
//...e
//...sgetHeaderNames:2:
public Enumeration<String> getHeaderNames()
  {
  return headers.keys();
  }
//...e
//...sgetRemoteUser:2:
public String getRemoteUser()
  {
  return userid;
  }
//...e
//...sisUserInRole:2:
public boolean isUserInRole(String role)
  {
  return userid != null && authenticator.isUserInRole(userid, role);
  }
//...e
  // other
//...sHttpServletRequest:2:
public HttpServletRequest(InputStream is, String scheme, int port)
  throws IOException
  {
  this.scheme = scheme;
  this.port   = port;
  InputStreamReader isr = new InputStreamReader(is);
  BufferedReader br = new BufferedReader(isr);
  String firstLine = br.readLine();
  if ( firstLine == null || firstLine.length() == 0 )
    // I've seen Chrome open speculative sockets over SSL and then close them
    // before actually issuing a request, so we report this differently
    // so the caller can elect to treat it differently (no point logging it)
    throw new EOFException("Request missing");
  StringTokenizer st = new StringTokenizer(firstLine);
  if ( ! st.hasMoreTokens() )
    throw new IOException("Request has no method");
  method = st.nextToken().toUpperCase();
  if ( ! st.hasMoreTokens() )
    throw new IOException("Request has no URI");
  uri = st.nextToken();
  int i;
  if ( (i = uri.indexOf('?')) != -1 )
    // The requestURI is everything upto the ?.
    {
    query = uri.substring(i+1);
    uri = uri.substring(0, i);
      // The version returned by getRequestURI is not URL decoded
      // However, the decoded version is used to match the servlet context
    }
  else
    query = "";
  if ( st.hasMoreTokens() )
    {
    rh = true;
    protocol = st.nextToken();
    }
  else
    {
    rh = false;
    protocol = "HTTP/1.0";
    }
  String line;
  while ( (line = br.readLine()) != null && line.length() != 0 )
    // Note: header names should be case INSENSITIVE, despite our use of a
    // Hashtable of Strings, which in effect means ours are case sensitive.
    {
    int inx;
    if ( (inx = line.indexOf(':')) != -1 )
      {
      String h = line.substring(0, inx);
      String v = line.substring(inx+1).trim();
      Vector<String> vec;
      if ( (vec = headers.get(h)) == null )
        {
        vec = new Vector<String>();
        headers.put(h, vec);
        }
      vec.add(v);
      }
    }
  // on a POST form, the parameters are in the body
  int length;
  if ( method.equals("POST") &&
       getHeader("Content-Type").equals("application/x-www-form-urlencoded") &&
       (length = getIntHeader("Content-Length")) != -1 )
    // note: we don't support "multipart/form-data"
    // the form data isn't terminated by a newline, so we can't br.readLine()
    // so we have to read a certain number of bytes
    // a problem is that we can't do byte level access on a BufferedReader,
    // and because its buffered, we can't simply suck on the InputStream
    // so for now, we assume 1 byte per character
    // for a GET, only ASCII characters are allowed
    // for a POST, all of ISO10646 is supposed to be supported
    {
    char[] buf = new char[length];
    if ( br.read(buf) != length )
      throw new IOException("Couldn't read enough bytes of POST data");
    query = new String(buf);
    }
  // now process the query string
  StringTokenizer st2 = new StringTokenizer(query, "&");
  while ( st2.hasMoreTokens() )
    {
    String token = st2.nextToken();
    int i2;
    String p, v;
    // request parameter names and values are URL decoded
    if ( (i2 = token.indexOf('=')) != -1 )
      {
      p = URLDecoder.decode( token.substring(0, i2), "UTF-8" );
      v = URLDecoder.decode( token.substring(i2+1 ), "UTF-8" );
      }
    else
      {
      p = URLDecoder.decode( token, "UTF-8" );
      v = "";
      }
    Vector<String> vec;
    if ( (vec = parameters.get(p)) == null )
      {
      vec = new Vector<String>();
      parameters.put(p, vec);
      }
    vec.add(v);
    }
  // authentication information may be added later
  userid        = null;
  authenticator = null;
  }
//...e
//...sattachUserid:2:
public void attachUserid(
  String userid,
  String authType,
  Authenticator authenticator
  )
  {
  this.userid        = userid;        // to support getRemoteUser()
  this.authType      = authType;      // to support getAuthType()
  this.authenticator = authenticator; // to support isUserInRole()
  }
//...e
//...sgetRespHdrs:2:
public boolean getRespHdrs()
  {
  return rh;
  }
//...e
  }
