//
// ServletEngine.java - Nyangau Servlet Engine
//

package nyangau.se;

//...simports:0:
import java.net.*;
import java.io.*;
import java.util.*;
import java.security.*;
import javax.net.*;
import javax.net.ssl.*;
import java.lang.management.*;
import javax.management.*;
//...e

public class ServletEngine
  {
  private static final String VERSION = "2.2";
  protected String server; // Server: response header, or "" if nothing
  protected ServletContext servletContext;
  protected Authenticator authenticator;
  protected Vector<RegisteredServlet> registeredServlets = new Vector<RegisteredServlet>();
//...slog:2:
protected void log(String message)
  {
  System.err.println(message);
  }
//...e
//...sServletEngine:2:
public ServletEngine(ServletContext sc)
  throws IOException
  {
  servletContext = sc;
  if ( (server = sc.getInitParameter("server")) == null )
    server = "nyangau.se "+VERSION;
  }
//...e
//...sregisterServlet:2:
public void registerServlet(
  String n, jakarta.servlet.http.HttpServlet s, ServletConfig sc
  )
  throws jakarta.servlet.ServletException, IOException
  {
  registeredServlets.add(new RegisteredServlet(n, s, sc));
  sc.setServletContext(servletContext);
  s.init(sc);
  }
//...e
//...sgetServletContext:2:
public ServletContext getServletContext()
  { return servletContext; }
//...e
//...sgetAuthenticator:2:
public Authenticator getAuthenticator()
  { return authenticator; }
//...e
//...sgetRegisteredServlets:2:
public Vector<RegisteredServlet> getRegisteredServlets()
  { return registeredServlets; }
//...e
//...sgetServer:2:
public String getServer()
  {
  return server;
  }
//...e
//...sserve:2:
// Net effect of the j_http(s) mechanism is that if we start any
// ConnectionServers this thread will block, but if we don't start any
// this thread returns to its caller (which they may find a shock).

//...stoInt:2:
protected static int toInt(String s, int def)
  {
  int v;
  try
    {
    v = Integer.parseInt(s);
    }
  catch ( NumberFormatException nfe )
    {
    v = def;
    }
  return v;
  }
//...e

public void serve()
  throws IOException
  {
  authenticator = new Authenticator(servletContext);
  Thread j_http = null;
  Thread j_https = null;
  String p;
  if ( (p = servletContext.getInitParameter("port")) != null )
    {
    int port = toInt(p, 8080);
    ServerSocket ss = new ServerSocket(port);
    ConnectionServer cs = new ConnectionServer(this, ss, "http", port);
    cs.start();
    j_http = cs;
    }
  if ( (p = servletContext.getInitParameter("port_ssl")) != null )
    {
    int port_ssl = toInt(p, 8443);
    String ksfn;
    if ( (ksfn = servletContext.getInitParameter("keystore")) == null )
      ksfn = "se.jks";
    String kspw;
    if ( (kspw = servletContext.getInitParameter("storepass")) == null )
      kspw = "password";
    try
      {
      KeyStore ks = KeyStore.getInstance("JKS");
      char[] ksp = kspw.toCharArray();
      try
        (
        InputStream kis = new FileInputStream(ksfn);
        )
        {
        ks.load(kis, ksp);
        }
      KeyManagerFactory kmf = KeyManagerFactory.getInstance(KeyManagerFactory.getDefaultAlgorithm());
      kmf.init(ks, ksp);
      KeyStore ts = KeyStore.getInstance("JKS");
      char[] tsp = kspw.toCharArray();
      try
        (
        InputStream tis = new FileInputStream(ksfn);
        )
        {
        ts.load(tis, tsp);
        }
      TrustManagerFactory tmf = TrustManagerFactory.getInstance(TrustManagerFactory.getDefaultAlgorithm());
      tmf.init(ts);
      SSLContext sc = SSLContext.getInstance("TLS");
      sc.init(kmf.getKeyManagers(), tmf.getTrustManagers(), null);
      ServerSocketFactory ssf = sc.getServerSocketFactory();
      ServerSocket ss = ssf.createServerSocket(port_ssl);
      ConnectionServer cs = new ConnectionServer(this, ss, "https", port_ssl);
      cs.start();
      j_https = cs;
      }
    catch ( GeneralSecurityException e )
      {
      log("Error setting up SSL: "+e.toString());
      }
    }
  try
    {
    if ( j_http != null )
      j_http.join();
    if ( j_https != null )
      j_https.join();
    }
  catch ( InterruptedException e )
    {
    // Won't happen, we never use Thread.interrupt()
    }
  }
//...e
  }
